/////////////////////////////////////////////////////////////////////////////////////
// File:	configfile.h
// Author:	Walt Jones
// Email:	wjones@blarg.net
// Date:	8 August, 2000
// Project:	ResNet Autoreg v2.0
// Purpose:	System configuration file object
/////////////////////////////////////////////////////////////////////////////////////

// Make sure this file is included only once
#if !defined( CONFIGFILE_H )
#define CONFIGFILE_H

#include <iostream>
#include <fstream>
#include <stdlib.h>
#include <string>
#include <vector>
#include <time.h>
#include "filelock.h"

// Return types definitions
typedef enum {
	configFileSuccess=1,
	configFileFail=0
} configFileResultType;

class configFileInterface
{
private:
	// Variables
	ifstream fin;			// Input filestream
	ofstream fout;			// Outfile filestream
	char *filename;			// Filename
	fileLock lock;			// File lock
	char *delims;			// Delimeters for tokenizer
	char *OShead;			// Header for the OS section
	char *MANhead;			// Header for the MAN section
	char *NIChead;			// Header for the NIC section
	char *HUBhead;			// Header for the HUB section
	char *SYSTEMhead;		// Header for the SYSTEM section
	char *DHCPCONFspec;		// Specifier for the DHCPCONF entry
	char *DHCPLEASEspec;		// Specifier for the DHCPLEASE entry
	char *ERROR_LOGspec;		// Specifier for the ERROR_LOG entry
	char *ACCESS_LOGspec;		// Specifier for the ACCESS_LOG entry
	char *POLICYspec;		// Specifier for the POLICY entry
	char *DBspec;			// Specifier for the DB entry

	// Static class variables
	static const int maxStringSize;
	static const int maxDigitStringSize;

	// Methods
	configFileResultType getMultiChoices(vector<string> &choices, const char section[]);
	configFileResultType getSystemEntry(string &value, const char entryName[]);
	configFileResultType updateVectorChoices(vector<string> &choices, const char section[]);
	configFileResultType updateStringEntry(string &value, const char entryName[]);
	configFileResultType writeFile(vector<string> &contents);
	void appendVect(vector<string> &output, vector<string> &input);

public:
	// Methods
	configFileInterface(const string &filename);
	~configFileInterface();
	configFileResultType open();
	void close();
	configFileResultType getOSchoices(vector<string> &choices) {return getMultiChoices(choices, OShead);};
	configFileResultType getMANchoices(vector<string> &choices) {return getMultiChoices(choices, MANhead);};
	configFileResultType getNICchoices(vector<string> &choices) {return getMultiChoices(choices, NIChead);};
	configFileResultType getHUBchoices(vector<string> &choices) {return getMultiChoices(choices, HUBhead);};
	configFileResultType getDHCPCONF(string &value) {return getSystemEntry(value, DHCPCONFspec);};
	configFileResultType getDHCPLEASE(string &value) {return getSystemEntry(value, DHCPLEASEspec);};
	configFileResultType getERROR_LOG(string &value) {return getSystemEntry(value, ERROR_LOGspec);};
	configFileResultType getACCESS_LOG(string &value) {return getSystemEntry(value, ACCESS_LOGspec);};
	configFileResultType getPOLICY(string &value) {return getSystemEntry(value, POLICYspec);};
	configFileResultType getDB(string &value) {return getSystemEntry(value, DBspec);};
	

	configFileResultType updateOSchoices(vector<string> &choices) {return updateVectorChoices(choices, OShead);};
	configFileResultType updateMANchoices(vector<string> &choices) {return updateVectorChoices(choices, MANhead);};
	configFileResultType updateNICchoices(vector<string> &choices) {return updateVectorChoices(choices, NIChead);};
	configFileResultType updateHUBchoices(vector<string> &choices) {return updateVectorChoices(choices, HUBhead);};
	configFileResultType updateDHCPCONF(string &value) {return updateStringEntry(value, DHCPCONFspec);};
	configFileResultType updateDHCPLEASE(string &value) {return updateStringEntry(value, DHCPLEASEspec);};
	configFileResultType updateERROR_LOG(string &value) {return updateStringEntry(value, ERROR_LOGspec);};
	configFileResultType updateACCESS_LOG(string &value) {return updateStringEntry(value, ACCESS_LOGspec);};
	configFileResultType updatePOLICY(string &value) {return updateStringEntry(value, POLICYspec);};
	configFileResultType updateDB(string &value) {return updateStringEntry(value, DBspec);};
};

// Initialize static class variables
const int configFileInterface::maxStringSize = 256;
const int configFileInterface::maxDigitStringSize = 13;

/////////////////////////////////////////////////////////////////////////////////////
// Method:	configFileInterface
// Purpose:	Constructor
// Input:	file - The filename (with full path) to the configuration file to use
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
configFileInterface::configFileInterface(const string &file)
{
	// Set up arrays
	filename = new char[maxStringSize + 1];
	delims = new char[2];

	// Set up the delimeter and the filenames
	delims[0] = '=';
	delims[1] = '\0';
	strcpy(filename, file.c_str());

	// Set up section headers
	OShead = new char[25];
	strcpy(OShead, "[OSCHOICE]");
	MANhead = new char[25];
	strcpy(MANhead, "[MANCHOICE]");
	NIChead = new char[25];
	strcpy(NIChead, "[NICCHOICE]");
	HUBhead = new char[25];
	strcpy(HUBhead, "[HUBCHOICE]");
	SYSTEMhead = new char[25];
	strcpy(SYSTEMhead, "[SYSTEM]");
	DHCPCONFspec = new char[25];
	strcpy(DHCPCONFspec , "DHCPCONF");
	DHCPLEASEspec = new char[25];
	strcpy(DHCPLEASEspec , "DHCPLEASE");
	ERROR_LOGspec = new char[25];
	strcpy(ERROR_LOGspec, "ERROR_LOG");
	ACCESS_LOGspec = new char[25];
	strcpy(ACCESS_LOGspec, "ACCESS_LOG");
	POLICYspec = new char[25];
	strcpy(POLICYspec, "POLICY");
	DBspec = new char[25];
	strcpy(DBspec, "DB");

	// Set up the fileLock
	lock.setFilename(filename);

	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	~configFileInterface
// Purpose:	Destructor
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
configFileInterface::~configFileInterface()
{
	// Close files
	close();
	fout.close();
	
	delete filename;
	delete delims;
	delete OShead;
	delete MANhead;
	delete NIChead;
	delete HUBhead;
	delete SYSTEMhead;
	delete DHCPCONFspec;
	delete DHCPLEASEspec;
	delete ERROR_LOGspec;
	delete ACCESS_LOGspec;
	delete POLICYspec;
	delete DBspec;
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	open
// Purpose:	Opens the filestream fin
// Input:	None
// Output:	None
// Return:	configFileSuccess - The file was opened successfully
//		configFileFail - An error occured
/////////////////////////////////////////////////////////////////////////////////////
configFileResultType configFileInterface::open()
{
	// Open the file
	lock.lock();
	fin.open(filename);
	if (fin.fail())
	{
		lock.release();
		return configFileFail;
	}
	return configFileSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	close
// Purpose:	Closes the filestream fin
// Input:	None
// Output:	None
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void configFileInterface::close()
{
	fin.close();
	if (lock.isLocked())
		lock.release();
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getMultiChoices
// Purpose:	Extracts the choices from a given section of the config file
// Input:	section - The section to get choices from
// Output:	choices - The choices from the section specified
// Return:	configFileSuccess - The choices were extracted successfully
//		configFileFail - An error occured
/////////////////////////////////////////////////////////////////////////////////////
configFileResultType configFileInterface::getMultiChoices(vector<string> &choices, const char section[])
{
	char templine[maxStringSize + 1];		// Temporary line
	
	// Reset the file buffer position
	filebuf filebuffer(fin.rdbuf()->fd());
	filebuffer.seekpos(0, ios::in);
	
	// Make sure the vector is empty
	if (!choices.empty())
		choices.clear();

	// Go to where the section begins
	do
	{
		fin.getline(templine, maxStringSize, '\n');
	}while (strcmp(templine, section) && (!fin.eof()));

	// If we're not in the right place, return configFileFail
	if (strcmp(templine, section))
	{
		close();
		return configFileFail;
	}

	// Get all of the entries
	while (!fin.eof())
	{
		// Get a line
		fin.getline(templine, maxStringSize, '\n');

		// If the line is length zero, we're done
		if (strlen(templine) == 0)
			break;

		// Add the choice to our list
		choices.push_back(templine);
	}
	return configFileSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	getSystemEntry
// Purpose:	Extracts the specified entry from the system section of the config file
// Input:	entryName - The name of the entry to get
// Output:	value - The value of the entry
// Return:	configFileSuccess - The timeout setting was extracted successfully
//			configFileFail - An error occured
/////////////////////////////////////////////////////////////////////////////////////
configFileResultType configFileInterface::getSystemEntry(string &value, const char entryName[])
{
	char templine[maxStringSize + 1];		// Temporary line
	char *token;			// Token

	// Reset the file buffer position
	filebuf filebuffer(fin.rdbuf()->fd());
	filebuffer.seekpos(0, ios::in);

	// Go to where the section begins
	do
	{
		fin.getline(templine, maxStringSize, '\n');
	}while ( strcmp(templine, SYSTEMhead) && (!fin.eof()) );

	// If we're not in the right place, return configFileFail
	if (strcmp(templine, SYSTEMhead))
	{
		close();
		return configFileFail;
	}

	// Go to the entry
	do
	{
		fin.getline(templine, maxStringSize, '\n');
		while ((strlen(templine) == 0)&&(!fin.eof()))	// If there's nothing in templine, keep getting new ones
			fin.getline(templine, maxStringSize, '\n');
		token = strtok(templine, delims);
	}while ( (strcmp(token, entryName)) && (!fin.eof()) );

	if (!strcmp(token, entryName))
	{
		token = strtok(NULL, delims);

		if (token != NULL)
			value = string(token);
		else	// If we don't have a token, return configFileFail
			return configFileFail;
	}
	else	// We didn't find what we wanted, return configFileFail
	{
		close();
		return configFileFail;
	}
	return configFileSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	appendVect
// Purpose:	Appends a vector to another vector
// Input:	input - The vector to append
// Output:	output - The vector to append the other vector to
// Return:	None
/////////////////////////////////////////////////////////////////////////////////////
void configFileInterface::appendVect(vector<string> &output, vector<string> &input)
{
	vector<string>::iterator end, it;	// Vector iterators

	// Go through the input vector and append it to the output vector
	end = input.end();
	for (it=input.begin(); it != end; it++)
		output.push_back(*it);
	return;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	writeFile
// Purpose:	Writes the contents of a vector to the config file
// Input:	contents - The vector to write to disk
// Output:	None
// Return:	configFileSuccess - The file was written successfully
//			configFileFail - An error occured
/////////////////////////////////////////////////////////////////////////////////////
configFileResultType configFileInterface::writeFile(vector<string> &contents)
{
	vector<string>::iterator end, it;	// Vector iterator

	// Open the output file
	lock.lock();
	fout.open(filename, ios::trunc);
	if (fout.fail())
	{
		close();
		return configFileFail;
	}

	// Write the contents of the vector to the file
	end = contents.end();
	for (it=contents.begin(); it != end; it++)
		fout << string(*it).c_str() << endl;

	fout.close();
	lock.release();
	return configFileSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	updateVectorChoices
// Purpose:	Updates the specified section of the config file with new selections from a vector
// Input:	choices - The choices to put in the specified section
//			section - The section to update
// Output:	None
// Return:	configFileSuccess - The config file was updated successfully
//			configFileFail - An error occured
/////////////////////////////////////////////////////////////////////////////////////
configFileResultType configFileInterface::updateVectorChoices(vector<string> &choices, const char section[])
{
	vector<string> tempVect;	// Temporary vector
	vector<string> contents;	// Contents of the file
	string tempString;			// Temporary string

	// Open the file for input
	open();
	
	if (strcmp(section, OShead))
	{
		// Get all the stuff in the OS section
		if (!getOSchoices(tempVect))
			return configFileFail;
		contents.push_back(OShead);
		appendVect(contents, tempVect);
		contents.push_back("\n");
	}
	else
	{
		// Update the OS section
		contents.push_back(OShead);
		appendVect(contents, choices);
		contents.push_back("\n");
	}

	if (strcmp(section, MANhead))
	{
		// Get all the stuff in the MAN section
		if (!getMANchoices(tempVect))
			return configFileFail;
		contents.push_back(MANhead);
		appendVect(contents, tempVect);
		contents.push_back("\n");
	}
	else
	{
		// Update the MAN section
		contents.push_back(MANhead);
		appendVect(contents, choices);
		contents.push_back("\n");
	}

	if (strcmp(section, NIChead))
	{
		// Get all the stuff in the NIC section
		if (!getNICchoices(tempVect))
			return configFileFail;
		contents.push_back(NIChead);
		appendVect(contents, tempVect);
		contents.push_back("\n");
	}
	else
	{
		// Update the NIC section
		contents.push_back(NIChead);
		appendVect(contents, choices);
		contents.push_back("\n");
	}

	if (strcmp(section, HUBhead))
	{
		// Get all the stuff in the HUB section
		if (!getHUBchoices(tempVect))
			return configFileFail;
		contents.push_back(HUBhead);
		appendVect(contents, tempVect);
		contents.push_back("\n");
	}
	else
	{
		// Update the HUB section
		contents.push_back(HUBhead);
		appendVect(contents, choices);
		contents.push_back("\n");
	}

	// If we were told to update the SYSTEM section, we can't
	if (!strcmp(section, SYSTEMhead))
	{
		close();
		return configFileFail;
	}

	// Get the stuff from the SYSTEM section
	contents.push_back(SYSTEMhead);
	if (!getDHCPCONF(tempString))
	{
		close();
		return configFileFail;
	}
	contents.push_back(string(DHCPCONFspec) + "=" + tempString);
	if (!getDHCPLEASE(tempString))
	{
		close();
		return configFileFail;
	}
	contents.push_back(string(DHCPLEASEspec) + "=" + tempString);
	if (!getERROR_LOG(tempString))
	{
		close();
		return configFileFail;
	}
	contents.push_back(string(ERROR_LOGspec) + "=" + tempString);
	if (!getACCESS_LOG(tempString))
	{
		close();
		return configFileFail;
	}
	contents.push_back(string(ACCESS_LOGspec) + "=" + tempString);
	if (!getPOLICY(tempString))
	{
		close();
		return configFileFail;
	}
	contents.push_back(string(POLICYspec) + "=" + tempString);
	if (!getDB(tempString))
	{
		close();
		return configFileFail;
	}
	contents.push_back(string(DBspec) + "=" + tempString);
	
	// Close the file
	close();

	// Write the new file
	if (!writeFile(contents))
	{
		close();
		return configFileFail;
	}

	return configFileSuccess;
}

/////////////////////////////////////////////////////////////////////////////////////
// Method:	updateStringEntry
// Purpose:	Updates the specified entry in the config file with a new value from a string
// Input:	value - The new value for the entry
//			entryName - The entry to update
// Output:	None
// Return:	configFileSuccess - The config file was updated successfully
//			configFileFail - An error occured
/////////////////////////////////////////////////////////////////////////////////////
configFileResultType configFileInterface::updateStringEntry(string &value, const char entryName[])
{
	vector<string> tempVect;	// Temporary vector
	vector<string> contents;	// Contents of the file
	string tempString;			// Temporary string

	// Open the file for input
	open();

	// Get all the stuff in the OS section
	if (!getOSchoices(tempVect))
		return configFileFail;
	contents.push_back(OShead);
	appendVect(contents, tempVect);
	contents.push_back("\n");

	// Get all the stuff in the MAN section
	if (!getMANchoices(tempVect))
		return configFileFail;
	contents.push_back(MANhead);
	appendVect(contents, tempVect);
	contents.push_back("\n");

	// Get all the stuff in the NIC section
	if (!getNICchoices(tempVect))
		return configFileFail;
	contents.push_back(NIChead);
	appendVect(contents, tempVect);
	contents.push_back("\n");

	// Get all the stuff in the HUB section
	if (!getHUBchoices(tempVect))
		return configFileFail;
	contents.push_back(HUBhead);
	appendVect(contents, tempVect);
	contents.push_back("\n");

	// Get the stuff from the SYSTEM section
	contents.push_back(SYSTEMhead);

	if (strcmp(entryName, DHCPCONFspec))
	{
		if (!getDHCPCONF(tempString))
		{
			close();
			return configFileFail;
		}
		contents.push_back(string(DHCPCONFspec) + "=" + tempString);
	}
	else
		contents.push_back(string(DHCPCONFspec) + "=" + value);
		
	if (strcmp(entryName, DHCPLEASEspec))
	{
		if (!getDHCPLEASE(tempString))
		{
			close();
			return configFileFail;
		}
		contents.push_back(string(DHCPLEASEspec) + "=" + tempString);
	}
	else
		contents.push_back(string(DHCPLEASEspec) + "=" + value);

	if (strcmp(entryName, ERROR_LOGspec))
	{
		if (!getERROR_LOG(tempString))
		{
			close();
			return configFileFail;
		}
		contents.push_back(string(ERROR_LOGspec) + "=" + tempString);
	}
	else
		contents.push_back(string(ERROR_LOGspec) + "=" + value);

	if (strcmp(entryName, ACCESS_LOGspec))
	{
		if (!getACCESS_LOG(tempString))
		{
			close();
			return configFileFail;
		}
		contents.push_back(string(ACCESS_LOGspec) + "=" + tempString);
	}
	else
		contents.push_back(string(ACCESS_LOGspec) + "=" + value);
		
	if (strcmp(entryName, POLICYspec))
	{
		if (!getPOLICY(tempString))
		{
			close();
			return configFileFail;
		}
		contents.push_back(string(POLICYspec) + "=" + tempString);
	}
	else
		contents.push_back(string(POLICYspec) + "=" + value);
	if (strcmp(entryName, DBspec))
	{
		if (!getDB(tempString))
		{
			close();
			return configFileFail;
		}
		contents.push_back(string(DBspec) + "=" + tempString);
	}
	else
		contents.push_back(string(DBspec) + "=" + value);

	// Close the file
	close();
	
	// Write the new file
	if (!writeFile(contents))
	{
		close();
		return configFileFail;
	}

	return configFileSuccess;
}

#endif // defined( CONFIGFILE_H )
