/////////////////////////////////////////////////////////////////////////////////////
// File:	ipfunctions.h
// Author:	Walt Jones
// Email:	wjones@blarg.net
// Date:	16 July, 2000
// Project:	ResNet Autoreg v2.0
// Purpose:	Functions that process IP addresses
/////////////////////////////////////////////////////////////////////////////////////

// Make sure this file is included only once
#if !defined( IPFUNCTIONS_H )
#define IPFUNCTIONS_H

#include <string>
#include <stdlib.h>
#include "stringfunc.h"

string getSubnet(const string &IP, const string &mask);
string formatIP(const string &IP);

/////////////////////////////////////////////////////////////////////////////////////
// Function:	getSubnet
// Purpose:		Determines the subnet of a given IP based on the subnet mask
// Input:		IP - The IP address to find the subnet of
//				mask - The subnet mask to use
// Output:		None
// Return:		The subnet of IP
/////////////////////////////////////////////////////////////////////////////////////
string getSubnet(const string &IP, const string &mask)
{
	char delims[2];		// Delimeters for the IP address and subnet mask
	int subnet[4];		// The subnet of the IP specified
	int IPint[4];		// Integer array version of the IP address
	int maskInt[4];		// Integer array version of the subnet mask
	char *IPchar;		// Character array version of the IP address
	char *maskChar;		// Character array version of the subnet mask
	string subnetStr;	// Subnet of IP as a string

	// Set up the variables we're going to use
	IPchar = new char[IP.length()];
	strcpy(IPchar, IP.c_str());
	maskChar = new char[mask.length()];
	strcpy(maskChar, mask.c_str());
	delims[0] = '.';
	delims[1] = '\0';

	// Break apart the IP
	IPint[0] = atoi(strtok(IPchar, delims));
	IPint[1] = atoi(strtok(NULL, delims));
	IPint[2] = atoi(strtok(NULL, delims));
	IPint[3] = atoi(strtok(NULL, delims));

	// Break apart the mask
	maskInt[0] = atoi(strtok(maskChar, delims));
	maskInt[1] = atoi(strtok(NULL, delims));
	maskInt[2] = atoi(strtok(NULL, delims));
	maskInt[3] = atoi(strtok(NULL, delims));

	// Determine the subnet by taking the bitwise AND of each element in the IP and subnet mask
	for(int i=0; i < 4; i++)
		subnet[i] = (IPint[i] & maskInt[i]);
	
	// Build the subnet string and return it
	subnetStr = formatIP(itostr(subnet[0], 10) + "." + itostr(subnet[1], 10) + "." + itostr(subnet[2], 10) + "." + itostr(subnet[3], 10));
	delete delims;
	delete subnet;
	delete IPint;
	delete maskInt;
	delete IPchar;
	delete maskChar;
	return subnetStr;
}

/////////////////////////////////////////////////////////////////////////////////////
// Function:	formatIP
// Purpose:		Formats an IP address in the form xxx.xxx.xxx.xxx with leading zeros
// Input:		IP - The IP address to format
// Output:		None
// Return:		The correctly formatted IP
/////////////////////////////////////////////////////////////////////////////////////
string formatIP(const string &IP)
{
	char delims[2];		// Delimeters for the IP address and subnet mask
	char IPchar[16];	// Character array version of the IP address
	char temp[4];		// Temporary character array
	string fmtIP;		// Correctly formatted IP address
	
	// Only do this if the IP isn't already in the right format
	if (IP.length() != 15)
	{
		// Set up the variables we're going to use
		strcpy(IPchar, IP.c_str());
		delims[0] = '.';
		delims[1] = '\0';

		// Break apart the IP and add leading zeros to each section before
		// putting it back together in fmtIP
		strcpy(temp, strtok(IPchar, delims));
		for (int i=0; i < 4; i++)
		{
			for (int j=3; j > strlen(temp); j--)
				fmtIP.append("0");
			fmtIP.append(temp);
			if (i < 3)
			{
				fmtIP.append(".");
				strcpy(temp, strtok(NULL, delims));
			}
		}
		delete delims;
		delete IPchar;
		delete temp;
		return fmtIP;
	}
	delete delims;
	delete IPchar;
	delete temp;
	return IP;	
}

#endif // defined( IPFUNCTIONS_H )
